#ifndef _T0_AREA_H
#define _T0_AREA_H

#include <Area.h>
#include <Piece.h>

/*
* This represents the column number, when a new area class is defined 
* it should be defined the second, third ... columns.
*/
#define FIRST_COLUMN 0

/*
* This number represents thath all the columns.
* The TN_ALL_COLUMNS should be defined in each TN_Area.
*/
#define T0_ALL_COLUMNS 1

class T0_Area : public Area {
  public:
  
    /*
    * The default constructor.
    */
    T0_Area();
    
    /*
    * The constructor.
    */
    T0_Area(void * _register_info);
    
    /*
    * The type 0 puzzle has only one column so, it does not matter the column
    * argument passed.
    * This method allocates the specified piece into this area.
    * Returns true if the piece was allocated. false otherwise.    
    */
    virtual bool allocate(Piece * piece, unsigned column);
    
    /*
    * This method returns the piece in the top.
    */
    Piece * get_up_piece(unsigned column);

    /*
    * This method returns the piece in the bottom.
    */
    Piece * get_down_piece(unsigned column);
    
    /*
    * This method returns the pattern of this Area.
    */ 
    unsigned get_pattern();
    
    /*
    * Returns a string representation of this Area.
    */
    virtual std::string to_string(
        DATA_STRING var_info_to_string,
        DATA_STRING register_info_to_string,
        bool show_register_info);
    
    /*
    * This method removes the pieces of this Area.
    */
    void remove_pieces();
    
    /*
    * This method returns true if the queried spots are free in the area:
    * Be carefull with the order taken, in the type n, n > 0
    * the parameters will be taken in reverse order.
    * You can pass: 
    *   no parameters -> in this case the the method will return true if the 
    *                    whole area is empty.
    *   one parameter -> in this case the method will return if the given row 
    *                    is free.
    *   both parameters -> the same as in the second case.
    */
    bool is_free(unsigned row, unsigned column);
    
    /*
    * Returns the number of free x spots.
    */
    unsigned get_free_x_number();
    
    /*
    * Returns the number of free z spots.
    */
    unsigned get_free_z_number();
    
  private:
    /*
    * The top piece.
    */
    Piece * top;
    /*
    * The bottom piece.
    */
    Piece * bottom;
    
};

#endif
