#include <map>
#include <Area.h>
#include <Board.h>
#include <string>

using namespace std;

/*
* The destructor calls the release_memory method, that method is virtual
* in order to be able to have a diferent behaivor in each subclass.
*/
Board::~Board(){
    release_memory();
}

void Board::clean_board() {
    unsigned size = areas.size();
    for(unsigned i = 0; i < size; i++) {
        areas[i]->remove_pieces();
    }
}

void Board::release_memory() {
    unsigned size = areas.size();
    for(unsigned i = 0; i < size; i++) {
        delete areas[i];
    }
    areas.clear();
}

/*
* Returns the number of x free spots.
*/
unsigned Board::get_free_x_number(){
    unsigned result = 0;
    unsigned size = areas.size();
    for(unsigned i = 0; i < size; i++) {
        result += areas[i]->get_free_x_number();
    }
    return result;
}
 
/*
* Returns the number of z free spots.
*/    
unsigned Board::get_free_z_number(){
    unsigned result = 0;
    unsigned size = areas.size();
    for(unsigned i = 0; i < size; i++) {
        result += areas[i]->get_free_z_number();
    }
    return result;
}

/*
* Returns an iterator to the areas biginning.
*/
std::vector<Area *>::iterator Board::get_areas_begin(){
    return areas.begin();
}

/*
* Returns an iterator to the areas biginning.
*/
std::vector<Area *>::iterator Board::get_areas_end(){
    return areas.end();
}

/*
* Returns a string representation of this Board.
*/
std::string Board::to_string(
DATA_STRING var_info_to_string,
DATA_STRING register_info_to_string
){
    vector<Area *>::iterator it;
    string rep = "";
    for(it = areas.begin();it!=areas.end();it++){
        rep += (*it)->to_string(var_info_to_string,register_info_to_string,true);
        rep += "\n";
    }
    return rep;
}
