#include <Piece.h>
#include <T0_Area.h>

#include <string>
using namespace std;

/*
* The default constructor.
*/
T0_Area::T0_Area():top(0),bottom(0){
    type = 0;
    register_info = 0;
}

/*
* The constructor.
*/
T0_Area::T0_Area(void * _register_info):top(0),bottom(0){
    type = 0;
    register_info = _register_info;
}

/**
* This method inserts the piece p into the puzzle area. The piece will cover
* the area completely.
* As it does not care the column parameter, you can call the method, without
* that parameter.
*/
bool T0_Area::allocate(Piece * piece, unsigned column = FIRST_COLUMN) {
    if( ! piece )
        return false;
  switch(piece->get_type()) {
    case X :
      if(top) {
        return false;
      }
      top = piece;
      break;
    case Y :
      if(top || bottom) {
        return false;
      }
      bottom = piece;
      top = piece;
      break;
    case Z :
      if(bottom) {
        return false;
      }
      bottom = piece;
      break;
  }
}

/*
* This method returns the piece in the top.
*/
Piece * T0_Area::get_up_piece(unsigned column = FIRST_COLUMN) {
    return top;
}

/*
* This method returns the piece in the bottom.
*/
Piece * T0_Area::get_down_piece(unsigned column = FIRST_COLUMN) {
    return bottom;
}

/*
* This method returns the pattern of this Area.
* top   bottom  pattern
* 0     0       0
* 0     1       1
* 1     0       2
* 1     1       3
*/ 
unsigned T0_Area::get_pattern(){
    unsigned pattern = (top)? 1 : 0;
    pattern *= 2;
    pattern += (bottom)? 1 : 0;
    return pattern;
}

/*
* Returns a string representation of this object. 
*/
std::string T0_Area::to_string(
DATA_STRING var_info_to_string,
DATA_STRING register_info_to_string,
bool show_register_info = true
) {
    std::string rep=
        (show_register_info)?
            "{" + register_info_to_string(register_info) + "} : "
            :"";
    rep += "[" ;
    rep += (top)?top->to_string(var_info_to_string):"";
    rep += "]/[" ;
    rep += (bottom)?bottom->to_string(var_info_to_string):"" ;
    rep += "]";
    return rep;
}

/*
* This method removes the pieces of this Area.
*/
void T0_Area::remove_pieces() {
    top = bottom = 0;
}

/*
* This method returns true if the queried spots are free in the area:
* Be carefull with the order taken, in the type n, n > 0
* the parameters will be taken in reverse order.
* You can pass: 
*   no parameters -> in this case the the method will return true if the whole
*                    area is empty.
*   one parameter -> in this case the method will return if the given row 
*                    is free.
*   both parameters ->  the same as in the second case, but it checks if the 
*                       values are in the right bound.
*/
bool T0_Area::is_free(unsigned row = 2, unsigned column = 1){
  if (column >1 || row > 2) //We don't want non valid parameters
    return false;   
  if (row == 0) {
    return top == NULL;
  } else if (row == 1){
    return bottom == NULL;
  }else{
    return top == NULL && bottom == NULL; 
  }
}

/*
* Returns the number of free x spots.
*/
unsigned T0_Area::get_free_x_number(){
    return (top)?0:1;
}

/*
* Returns the number of free z spots.
*/
unsigned T0_Area::get_free_z_number(){
    return (bottom)?0:1;
}
