#include <T0_Solver.h>
#include <spans.h>
#include <T0_Area.h>
#include <T0_patterns.h>

/*
* The constructor.
*/
T0_Solver::T0_Solver(Board * board_,PieceFactory * pieces_):Solver(board_,pieces_){}

bool T0_Solver::solve(){
    /*
    * First We have to padd.
    */
    pad();
    /*
    * The needed iterators.
    */
    std::vector<Area *>::iterator it;
    std::vector<Area *>::iterator end = board->get_areas_end();
    /*
    * 10: the upper row is taken by pre-colored registers.
    */
    for(it = board->get_areas_begin();it != end ;it++){
        //Check if the area has the 10 pattern
        Area * area = *it;
        if( area -> get_pattern() == T0_10 && 
            pieces->get_available_number(Z,
                area->get_preferences_begin(),
                area->get_preferences_end(),
                SPAN_1) >= 1){
            // Take the piece
            area->allocate(pieces->get_piece(Z,
                area->get_preferences_begin(),
                area->get_preferences_end(),
                SPAN_1),FIRST_COLUMN);
        }
    }
    
    /*
    * 01: the lower row is taken by pre-colored registers.
    */
    for(it = board->get_areas_begin();it != end ;it++){
        //Check if the area has the 01 pattern
        Area * area = *it;
        if( area -> get_pattern() == T0_01 && 
            pieces->get_available_number(X,
                area->get_preferences_begin(),
                area->get_preferences_end(),
                SPAN_1) >= 1){
            // Take the piece
            area->allocate(pieces->get_piece(X,
                area->get_preferences_begin(),
                area->get_preferences_end(),
                SPAN_1),FIRST_COLUMN);
        }
    }
    
    /*
    * 00: the area is empty. Try first filling it with
    * a Y piece, and if not possible, with a X and a Z pieces.
    */
    for(it = board->get_areas_begin();it != end ;it++){
        //Check if the area has the 00 pattern
        Area * area = *it;
        if( area -> get_pattern() == T0_00){
            if(pieces->get_available_number(Y,
                area->get_preferences_begin(),
                area->get_preferences_end(),
                SPAN_1) >= 1) {
                area->allocate(pieces->get_piece(Y,
                area->get_preferences_begin(),
                area->get_preferences_end(),
                SPAN_1),FIRST_COLUMN);
            }else if(pieces->get_available_number(X,
                area->get_preferences_begin(),
                area->get_preferences_end(),
                SPAN_1) >= 1 &&
                pieces->get_available_number(Z,
                area->get_preferences_begin(),
                area->get_preferences_end(),
                SPAN_1) >= 1){
                area->allocate(pieces->get_piece(X,
                area->get_preferences_begin(),
                area->get_preferences_end(),
                SPAN_1),FIRST_COLUMN);
                area->allocate(pieces->get_piece(Z,
                area->get_preferences_begin(),
                area->get_preferences_end(),
                SPAN_1),FIRST_COLUMN);
            }
        }
    }
    
    return pieces->get_available_number() == 0;
}
