#include <Piece.h>
#include <Area.h>
#include <T0_Area.h>
#include <T1_Area.h>

/*
* Constructor with all information.
*/
T1_Area::T1_Area(void * register_info_, Area * left_, Area * right_)
	:T2n_Area(1,register_info_,left_,right_){}

/*
* This method allocates the piece p into the given column of the puzzle area.
* If the second parameter is not given, the method will try to 
* allocate the piece in the left side.
*/
bool T1_Area::allocate(Piece * piece, unsigned column = FIRST_COLUMN) {
    //We will check default both rows.
    PieceType type = piece -> get_type();
    unsigned rows =     (type == X )? TOP_ROW : 
                        (type == Z )? BOTTOM_ROW:
                        BOTH_ROWS;
    if(column == 0){
        if(piece->get_size()==2){
            if( left->is_free(T1_ALL_COLUMNS,rows) && 
                right->is_free(T1_ALL_COLUMNS,rows)){
                return left->allocate(piece,FIRST_COLUMN) && 
                    right->allocate(piece,FIRST_COLUMN);
            }else{
                return false;
            }
        }else{
            return left->allocate(piece,FIRST_COLUMN);
        }
    }else{
        return right->allocate(piece,FIRST_COLUMN);
    }
}

/*
* Returns the piece that is allocated in the given column of the upper row of
* this area.
* If the column is not given, the method returns the piece in the first column.
*/
Piece * T1_Area::get_up_piece(unsigned column = FIRST_COLUMN) {
  if(column == FIRST_COLUMN) {
    return left->get_up_piece(column);
  } else {
    return right->get_up_piece(column);
  }
}

/*
* Returns the piece that is allocated in the given column of the lower row of
* this area.
* If the column is not given, the method returns the piece in the first column.
*/
Piece * T1_Area::get_down_piece(unsigned column = FIRST_COLUMN) {
  if(column == FIRST_COLUMN) {
    return left->get_down_piece(column);
  } else {
    return right->get_down_piece(column);
  }
}

/*
* Returns true if this area has some free spots. The integer parameters
* are the coordinates of the matrix that the area represents.
*/
bool T1_Area::is_free(
unsigned column = T1_ALL_COLUMNS, 
unsigned row = BOTH_ROWS) {
    if(column > T1_ALL_COLUMNS || row > BOTH_ROWS)
        return false;
    if ( column == FIRST_COLUMN) {
        return left->is_free(row,FIRST_COLUMN);
    } else if ( column == SECOND_COLUMN) {
        return right->is_free(row,FIRST_COLUMN);
    } else{
        return left->is_free(row,FIRST_COLUMN) && 
            right->is_free(row,FIRST_COLUMN);
    }
}
