#include <T2n_Area.h>

/*
* The constructor.
*/
T2n_Area::T2n_Area(
	unsigned type_,
    void * register_info_,
    Area * left_,
    Area * right_):left(left_),right(right_){
    type = type_;
    register_info = register_info_;
}

/*
* Removes all the pieces allocated in this area.
*/
void T2n_Area::remove_pieces() {
	left->remove_pieces();
	right->remove_pieces();
}

/*
* This method returns the pattern of this Area.
* This method requires to know the type puzzle, and to do that 
* eficently and to do not require more changes, it uses the type 
* member of the area class.
*/
unsigned T2n_Area::get_pattern(){
    return left->get_pattern() << ((unsigned)pow(2,type)) + right-> get_pattern();
}

/*
* Returns a string representation of this object. 
* This method is virtual in order to be able to choose another 
* string representation, but it can be used anyway.
*/
string T2n_Area::to_string(
DATA_STRING var_info_to_string,
DATA_STRING register_info_to_string,
bool show_register_info = true
) {
    string rep=
        (show_register_info)?
            "{" + register_info_to_string(register_info) + "} : "
            :"";
    rep += "[" ;
    rep += (left)? left->to_string(
        var_info_to_string,
        register_info_to_string,
        show_register_info):"" ;
    rep += "]/[";
    rep += (right)? right->to_string(
        var_info_to_string,
        register_info_to_string,
        show_register_info):"" ;
    rep += "]";
    return rep;
}

/*
* Returns the number of free x spots.
*/
unsigned T2n_Area::get_free_x_number(){
    return left->get_free_x_number() + right->get_free_x_number();
}

/*
* Returns the number of free z spots.
*/
unsigned T2n_Area::get_free_z_number(){
    return left->get_free_z_number() + right->get_free_z_number();
}

