#ifndef _AREA_H
#define _AREA_H

#include <Types.h>
#include <Piece.h>

#include <vector>

using namespace std;

/*
* This is to make code more understandable
*/
#define TOP_ROW 0
#define BOTTOM_ROW 1
#define BOTH_ROWS 2

/*
* This class represents an abstract puzzle area.
*/
class Area {
  
  public:
  
    /*
    * A virtual destructor is necesary to use pointers to base class.
    */
    virtual ~Area();
       
    /*
    * Returns a string representation of this Area.
    * It takes the DATA_STRING parameters and uses them to 
    * get the string representation of the pieces' var_info
    * and the area's register_info.
    */
    virtual std::string to_string(
    DATA_STRING var_info_to_string,
    DATA_STRING register_info_to_string,
    bool show_register_info
    )=0;
    
    /*
    * This method allocate a Piece in the given column.
    * Returns true if the piece was allocated. false otherwise.
    */
    virtual bool allocate(Piece * p, unsigned column) = 0;
    
    /*
    * This method return the piece allocated in the top row of this Area.
    */
    virtual Piece * get_up_piece(unsigned column) = 0;
    
    /*
    * This method return the piece allocated in the bottom row of this Area.
    */    
    virtual Piece * get_down_piece(unsigned column) = 0;    
    
    /*
    * This method returns the pattern of this area.
    * To represent the patern is more than enough use a unsigned, just remember 
    * the measauring unit is a byte.
    */
    virtual unsigned get_pattern() = 0;
    
    /*
    * This method removes all the pieces allocated in the area.
    */
    virtual void remove_pieces() = 0;
    
    /*
    * Returns the information of the register wich this area represents.
    */
    void * get_register_info();
    
    /*
    * This method returns true if the given coordinate (x,y) is free.
    * If the y parameter is not given, the method will return 
    * true if the whole column is free.
    * If both parameters are not given the method will return 
    * true if the whole area is free.
    */
    virtual bool is_free(unsigned x, unsigned y) = 0;
    
    /*
    * This method returns the type of this area.
    */
    unsigned get_type();
    
    /*
    * Adds the specified preference to this area.
    * This method should be called in the same order you want to 
    * look for the preference.
    */
    void add_preference(unsigned preference);
    
    /*
    * Returns an iterator to the beginning of the preferences.
    */
    vector<unsigned>::iterator get_preferences_begin();
    
    /*
    * Returns an iterator to the end of the preferences.
    */
    vector<unsigned>::iterator get_preferences_end();
    
    /*
    * Returns the number of free x spots.
    */
    virtual unsigned get_free_x_number() =0;
    
    /*
    * Returns the number of free z spots.
    */
    virtual unsigned get_free_z_number() =0;
    
  protected:
    
    /*
    * The information of the  register represented for this area.
    */
    void * register_info;
    
    /*
    * The area type.
    * Can be used for some algorithm.
    */
    unsigned type;
    
    /*
    * The vector of peferences.
    * This vector will guide the register allocation to use 
    * the apropiate pieces for each area.
    */
    std::vector<unsigned> preferences;
};

#endif
