#ifndef _BOARD_H
#define _BOARD_H

#include <map>
#include <vector>
#include <Area.h>
#include <Types.h>

class Board {
  public:
    /*
    * Returns a string representation of this Board.
    */
    virtual std::string to_string (
    DATA_STRING var_info_to_string,
    DATA_STRING register_info_to_string
    );
    
    /*
    * The destructor calls the release_memory method, that method is virtual
    * in order to be able to have a diferent behaivor in each subclass.
    */
    virtual ~Board();
    
    /*
    * This methos clean the board.
    */
    virtual void clean_board();
    
    /*
    * Returns the number of x free spots.
    */
    unsigned get_free_x_number();
    
    /*
    * Returns the number of z free spots.
    */    
    unsigned get_free_z_number();
    
    /*
    * Returns an iterator to the areas biginning.
    */
    std::vector<Area *>::iterator get_areas_begin();
    
    /*
    * Returns an iterator to the areas biginning.
    */
    std::vector<Area *>::iterator get_areas_end();
    
  protected:
    
    /*
    * Here We wiĺl store the Areas.
    */
    std::vector<Area *> areas;
    
    /*
    * This method frees the memory.
    * It is used in the destructor method.
    */
    void release_memory();
};

#endif
