#include <string>
#include <cstdlib>
#include <ctime>
#include <sstream>
#include <iostream>
#include <stdio.h>

#include <PieceType.h>
#include <T0_Solver.h>
#include <T1_Solver.h>
#include <spans.h>
#include <preferences.h>

#include <T0_X86_Board.h>
#include <T1_X86_Board.h>
#include <data_string.h>

#define T0_1_1 0x2
#define T0_1_2 0x1

#define T1_1_1 0x8
#define T1_1_2 0x4
#define T1_2_1 0x2
#define T1_2_2 0x1

using namespace std;

int main (int argc, char **argv ){
    //The board the pieces and the solver    
    T0_X86_Board * t0_board = new T0_X86_Board();
    PieceFactory * t0_pieces = new PieceFactory();
    T0_Solver * t0_solver = new T0_Solver(t0_board,t0_pieces);
    T1_X86_Board * t1_board = new T1_X86_Board();
    PieceFactory * t1_pieces = new PieceFactory();
    T1_Solver * t1_solver = new T1_Solver(t1_board,t1_pieces);
    
    //We will store here the pointers to delete
    vector<string *> strings_to_del;
    
    //We define a piece to allocate when the board is precolored.
    string * var_info_pre = new string("PRE");
    strings_to_del.push_back(var_info_pre);
    
    Piece fill_x(var_info_pre,X,0,0);
    Piece fill_z(var_info_pre,Z,0,0);
    
    //We aply a mask to know wich squares will be precolored.
    //We will need an iterator in order to pre color all areas in the board
    vector<Area *>::iterator it;
    //The current area.
    Area * area;
    //First we create the seed.
    srand(time(NULL));
    //The type 0 areas.
    unsigned random;
    
    for(it = t0_board->get_areas_begin();it!=t0_board->get_areas_end();it++){
        //Generate a random number and use it to pre color the board.
        random = rand();
        area = *it;
        if(random & T0_1_1)
            area->allocate(&fill_x,1);
        if(random & T0_1_2)
            area->allocate(&fill_z,1);
    }
    //The type 1 areas.
    /*
    for(it = t1_board->get_areas_begin();it!=t1_board->get_areas_end();it++){
        //Generate a random number and use it to pre color the board.
        random = rand();
        area = *it;
        if(random & T1_1_1)
            area->allocate(&fill_x,1);
        if(random & T1_1_2)
            area->allocate(&fill_z,1);
        if(random & T1_2_1)
            area->allocate(&fill_x,2);
        if(random & T1_2_2)
            area->allocate(&fill_z,2);
    }*/
    //A counter
    unsigned i;
    //A buffer to convert the subindex (when 0, a segmentation fault happens)
    char buff[10];
    //We will create at most 2 x pieces 2 y pieces and 2 z pieces
    random = rand() % 3 ;
    for (i = 0 ; i < random ; i++){
        ostringstream oss;
        sprintf(buff,"%u",i);
        oss << "x_" << buff;
        string * name = new string(oss.str());
        strings_to_del.push_back(name);
        t0_pieces->new_piece(name,X,16,1,DEFAULT_PREFERENCE,SPAN_1);
    }
    
    random = rand() % 3 ;
    for (i = 0 ; i < random ; i++){
        ostringstream oss;
        sprintf(buff,"%u",i);
        oss << "y_" << buff;
        string * name = new string(oss.str());
        strings_to_del.push_back(name);
        t0_pieces->new_piece(name,Y,16,1,DEFAULT_PREFERENCE,SPAN_1);
    }
    
    random = rand() % 3 ;
    for (i = 0 ; i < random ; i++){
        ostringstream oss;
        sprintf(buff,"%u",i);
        oss << "z_" << buff;
        string * name = new string(oss.str());
        strings_to_del.push_back(name);
        t0_pieces->new_piece(name,Z,16,1,DEFAULT_PREFERENCE,SPAN_1);
    }
    
    //We will create at most 3 x pieces, 1 y piece, 3 z pieces
    // 1 X piece 1 Y piece and 1 Z piece.
    
    random = rand() % 4 ;
    for (i = 0 ; i < random ; i++){
        ostringstream oss;
        sprintf(buff,"%u",i);
        oss << "x_" << buff;
        string * name = new string(oss.str());
        strings_to_del.push_back(name);
        t1_pieces->new_piece(name,X,8,1,DEFAULT_PREFERENCE,SPAN_1);
    }
    
    if(rand()%2){
        string * name = new string("y_0");
        strings_to_del.push_back(name);
        t1_pieces->new_piece(name,Y,8,1,DEFAULT_PREFERENCE,SPAN_1);
    }
    
    random = rand() % 4 ;
    for (i = 0 ; i < random ; i++){
        ostringstream oss;
        sprintf(buff,"%u",i);
        oss << "z_" << buff;
        string * name = new string(oss.str());
        strings_to_del.push_back(name);
        t1_pieces->new_piece(name,Z,8,1,DEFAULT_PREFERENCE,SPAN_1);
    }
    
    if(rand()%2){
        string * name = new string("X_0");
        strings_to_del.push_back(name);
        t1_pieces->new_piece(name,X,8,1,DEFAULT_PREFERENCE,SPAN_2);
    }
    
    if(rand()%2){
        string * name = new string("Y_0");
        strings_to_del.push_back(name);
        t1_pieces->new_piece(name,Y,8,1,DEFAULT_PREFERENCE,SPAN_2);
    }
    
    if(rand()%2){
        string * name = new string("Z_0");
        strings_to_del.push_back(name);
        t1_pieces->new_piece(name,Z,8,1,DEFAULT_PREFERENCE,SPAN_2);
    }
    
    cout << ">>>>>>>>>>>>>>>>>>>>>>>>>>>> t0_board <<<<<<<<<<<<<<<<<<<<<<<<<\n";
    //Print the t0 board
    cout << t0_board->to_string(t0_data_to_string,t0_data_to_string);
    //Print the t0 pieces
    cout << ">>>>>>>>>>>>>> all t0_pieces before the pad<<<<<<<<<<<<<<<<<<<<\n";
    cout << t0_pieces->all_to_string(t0_data_to_string) << endl;
    //Pad the board, it is not necesary to do it here.
    t0_solver->pad();
    cout << ">>>>>>>>>>>>>> all t0_pieces after the pad<<<<<<<<<<<<<<<<<<<<\n";
    cout << t0_pieces->all_to_string(t0_data_to_string) << endl;
    //Solve the t0 board
    cout << ">>>>>>>>>>>>>>>>>>>>>>>>>> solving <<<<<<<<<<<<<<<<<<<<<<<<<<<<\n";
    t0_solver->solve();
    //Print the used pieces
    cout << ">>>>>>>>>>>>>>>>>>>>>>>> t0 used_pieces <<<<<<<<<<<<<<<<<<<<<<<\n";
    cout << t0_pieces->used_to_string(t0_data_to_string) << endl;
    //Print the solved t0 board
    cout << ">>>>>>>>>>>>>>>>>>>>>> t0 solved board <<<<<<<<<<<<<<<<<<<<<<<<\n";
    cout << t0_board->to_string(t0_data_to_string,t0_data_to_string);    
    
    
    cout << ">>>>>>>>>>>>>>>>>>>>>>>>>>>> t1_board <<<<<<<<<<<<<<<<<<<<<<<<<\n";
    //Print the t1 board
    cout << t1_board->to_string(t1_data_to_string,t1_data_to_string);
    //Print the t1 pieces
    cout << ">>>>>>>>>>>>>> all t0_pieces before the pad<<<<<<<<<<<<<<<<<<<<\n";
    cout << t1_pieces->all_to_string(t1_data_to_string) << endl;
    //Pad the board, it is not necesary to do it here.
    t1_solver->pad();
    cout << ">>>>>>>>>>>>>> all t1_pieces after the pad<<<<<<<<<<<<<<<<<<<<\n";
    cout << t1_pieces->all_to_string(t1_data_to_string) << endl;
    //Solve the t1 board
    cout << ">>>>>>>>>>>>>>>>>>>>>>>>>> solving <<<<<<<<<<<<<<<<<<<<<<<<<<<<\n";
    t1_solver->solve();
    //Print the used pieces
    cout << ">>>>>>>>>>>>>>>>>>>>>>>> t1 used_pieces <<<<<<<<<<<<<<<<<<<<<<<\n";
    cout << t1_pieces->used_to_string(t1_data_to_string) << endl;
    //Print the solved t1 board
    cout << ">>>>>>>>>>>>>>>>>>>>>> t1 solved board <<<<<<<<<<<<<<<<<<<<<<<<\n";
    cout << t1_board->to_string(t1_data_to_string,t1_data_to_string);    

    //Delete all created objects
    for ( i = 0 ; i < strings_to_del.size() ; i++){
        delete strings_to_del[i];
    }
    strings_to_del.clear();
    delete t0_pieces;
    delete t0_board;
    delete t0_solver;
    delete t1_pieces;
    delete t1_board;
    delete t1_solver;
}
